from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django_idshost.models import IdsUser


class IdsUserForm(forms.ModelForm):

    confirm_password = forms.CharField(widget=forms.PasswordInput,
                                       label=_("Confirm the password."),
                                       required=True)
    with_named_cert = forms.BooleanField(initial=False, required=False)

    class Meta:
        model = IdsUser
        fields = ['username', 'email', 'password', 'confirm_password',
                  'with_named_cert']

    def __init__(self, *args, **kwargs):
        super(IdsUserForm, self).__init__(*args, **kwargs)
        if self.fields.get('username'):
            self.fields['username'].help_text = ''
        if self.fields.get('email'):
            self.fields['email'].required = True
        if self.fields.get('password'):
            self.fields['password'].widget = forms.PasswordInput()
            self.fields['password'].label = _("Password")

    def clean(self, *args, **kwargs):
        cleaned_data = super(IdsUserForm, self).clean(*args, **kwargs)
        pwd = cleaned_data.get('password')
        conf_pwd = cleaned_data.get('confirm_password')
        if pwd and conf_pwd and pwd != conf_pwd:
            self.errors['confirm_password'] = self.error_class(
                [_('Different')])
        return self.cleaned_data

    def save(self, commit=True):
        mask = 8 if self.cleaned_data['with_named_cert'] else None
        user = get_user_model(
        ).objects.create_user(self.cleaned_data['username'],
                              mask, self.cleaned_data['email'],
                              self.cleaned_data['password'],
                              commit=commit)
        return user



class Request(object):

    def __init__(self, application, requester, cookie, comment, *args,
                 **kwargs):
        self.application = application
        self.requester = requester
        self.cookie = cookie
        self.comment = comment

    def serialize_to_dict(self):
        return {
            'Application': self.application,
            'Requester': self.requester,
            'AuthCookie': self.cookie,
            'Comment': self.comment
        }


class GenerateCertificateRequest(Request):

    def __init__(self, application, requester, cookie, comment, ou, owner,
                 identifier, duration, mask, privilege=0, profile=0,
                 number=1, *args, **kwargs):

        super(GenerateCertificateRequest, self).__init__(application,
                                                         requester, cookie,
                                                         comment, *args,
                                                         **kwargs)
        self.ou = ou
        self.owner = owner
        self.identifier = identifier
        self._duration = duration
        self.mask = mask
        self.privilege = privilege
        self.profile = profile
        self.number = number

    @property
    def duration(self):
        if self._duration < 1 or self._duration > 1095:
            raise Exception("The duration must be less than 1095 and greater"
                            " than 0.")

        return self._duration

    def serialize_to_dict(self):
        parent = super(GenerateCertificateRequest, self).serialize_to_dict()
        parent.update({
            'OrganizationUnit': self.ou,
            'Owner': self.owner,
            'Identifier': self.identifier,
            'Privilege': self.privilege,
            'Profile': self.profile,
            'Duration': self.duration,
            'AuthenticationMask': self.mask,
            'Number': self.number
        })
        return parent


class RevokeCertificateRequest(Request):

    def __init__(self, application, requester, cookie, comment, ou, owner,
                 index, *args, **kwargs):
        super(RevokeCertificateRequest, self).__init__(application, requester,
                                                       cookie, comment)
        self.ou = ou
        self.owner = owner
        self.index = index

    def serialize_to_dict(self):
        parent = super(RevokeCertificateRequest, self).serialize_to_dict()
        parent.update({
            'OrganizationUnit': self.ou,
            'Owner': self.owner,
            'Index': self.index
        })
        return parent


class GetCertificateRequest(Request):

    def __init__(self, application, requester, cookie, owner_filter,
                 ou_filter, index_filter, requester_filter):
        super(GetCertificateRequest, self).__init__(application, requester,
                                                    cookie, '')
        self.owner_filter = owner_filter
        self.ou_filter = ou_filter
        self.index_filter = index_filter
        self.requester_filter = requester_filter

    def serialize_to_dict(self):
        parent = super(GetCertificateRequest, self).serialize_to_dict()
        del parent['Comment']
        parent.update({
            'OwnerFilter': self.owner_filter,
            'OrganizationUnitFilter': self.ou_filter,
            'IndexFilter': self.index_filter,
            'RequesterFilter': self.requester_filter
        })
        return parent



from django.conf.urls import url, patterns
from django.conf.urls.defaults import patterns, url


urlpatterns = patterns(
    'django_idshost.views',
    url(r'^logout_ids/', 'logout_ids',
        name='ids_disconnect'),
    url(r'^logout/', 'logout', name='logout'),
)



from django.db import models
from django.contrib.auth.models import AbstractUser, UserManager
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.signals import user_logged_in, user_logged_out
from django.dispatch import receiver


class IdsManager(UserManager):

    def create_user(self, username, cert_mask=None, email=None, password=None,
                    commit=True, **extra_fields):
        user = super(IdsManager, self).create_user(username, email, password,
                                                   **extra_fields)

        auth = '03{name}'.format(name=username)
        user.ids_authentifier = auth
        if commit:
            user.save(using=self._db)
        return user


class IdsUser(AbstractUser):

    ids_authentifier = models.CharField(
        _('ids authentifier'), max_length=100, unique=True,
        null=True, blank=True)
    REQUIRED_FIELDS = ['ids_authentifier']
    objects = IdsManager()



from django.db.utils import IntegrityError
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth import authenticate, get_user_model
from django.contrib.auth.views import logout as main_logout
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.conf import settings

from spyne.server.django import DjangoApplication
from spyne.server.wsgi import WsgiApplication
from spyne.model.primitive import String, Boolean, Unicode, Integer
from spyne.model.complex import Iterable
from spyne.service import ServiceBase
from spyne.protocol.soap import Soap11
from spyne.application import Application
from spyne.decorator import rpc
from spyne.interface.wsdl import Wsdl11


SETTINGS = getattr(settings, 'DJANGO_IDSHOST_SETTINGS', {})

if not SETTINGS.get('APP_NAME') or not SETTINGS.get('PRIVATE_IP'):
    raise Exception("You have to init the django_idshost settings to use"
                    " this app. Read the doc.")
else:
    app = SETTINGS.get('APP_NAME')
    private_ip = SETTINGS.get('PRIVATE_IP')
    public_url = SETTINGS.get('PUBLIC_URL', 'http://{app}'.format(app=app))


class AuthIds(ServiceBase):

    @rpc(String, String, String, _out_variable_name='IsValid',
         _returns=Boolean)
    def CheckPassword(ctx, OrganizationUnit, Authentifier, Password):
        res = False
        user = get_user_model().objects.get(username=Authentifier[2:])
        if user and user.check_password(Password):
            res = True
        return res


_app = Application([AuthIds],
                   '{public_url}'.format(public_url=public_url),
                   in_protocol=Soap11(validator='lxml'),
                   out_protocol=Soap11(),
                   name='idscheckpasswordservice',

                   )

_django_app = DjangoApplication(_app)
_django_app.doc.wsdl11.build_interface_document(
    "http://{private}/CheckPassword/".format(private=private_ip)
)

django_app = csrf_exempt(_django_app)


def _logout_url(request, next_page):

    protocol = 'https://'
    host = request.META.get('HTTP_HOST')
    url = protocol + host + next_page
    return url


def logout(request, **kwargs):
    if request.COOKIES.get('sessionids'):
        next_page = _logout_url(request, reverse('ids_disconnect'))
    else:
        prot = 'https://' if request.is_secure() else 'http://'
        next_page = prot + request.META.get('HTTP_HOST')
    return main_logout(request, next_page=next_page, **kwargs)


def logout_ids(request, **kwargs):
    return render(request, 'api/logout_ids.html',
                  {'auth': request.user.ids_authentifier})



from django.contrib.auth.middleware import RemoteUserMiddleware
from django.contrib.auth.backends import RemoteUserBackend, ModelBackend


class IdsRemoteUserBackend(RemoteUserBackend):

    def clean_username(self, username):
        return username[2:]


class IdsHeaderMiddleware(RemoteUserMiddleware):
    header = 'HTTP_IDS_USER'



from suds import WebFault
from suds.client import Client

from django_idshost.certificate.request import (GenerateCertificateRequest,
                                                RevokeCertificateRequest,
                                                GetCertificateRequest)


class ClientPki(object):

    wsdl = 'http://api.idshost.priv/pki.wsdl'

    def __init__(self, *args, **kwargs):
        super(ClientPki, self).__init__(*args, **kwargs)
        self.client = Client(ClientPki.wsdl)

    def _send_request(self, service, request, *args, **kwargs):
        try:
            return service(request.serialize_to_dict())
        except WebFault, e:
            raise e

    def generate_certificate(self, request, *args, **kwargs):
        if not isinstance(request, GenerateCertificateRequest):
            raise Exception("To send a generate request your request argument"
                            " have to be an instance of"
                            " GenerateCertificateRequest.")

        service = self.client.service.CertRequest
        return self._send_request(service, request, *args, **kwargs)

    def revoke_certificate(self, request, *args, **kwargs):
        if not isinstance(request, RevokeCertificateRequest):
            raise Exception("To send a revoke request your request argument"
                            " have to be an instance of"
                            " RevokeCertificateRequest.")
        service = self.client.service.CertRevocate
        return self._send_request(service, request, *args, **kwargs)

    def get_certificate(self, request, *args, **kwargs):
        if not isinstance(request, GetCertificateRequest):
            raise Exception("To send a get request your request argument"
                            " have to be an instance of"
                            " RevokeCertificateRequest.")
        service = self.client.service.GetCertificateRetrievalStatus
        return self._send_request(service, request, *args, **kwargs)



class Certificate(object):

    auth_method = {
        'sso': 1,
        'cps': 2,
        'ids': 4,
        'named': 8,
        'collective': 16,
        'automate': 32,
        'anonyme': 64,
        'ca': 128,
        'weak': 256
    }

    @staticmethod
    def build_mask(sso=False, cps=False, ids=False, named=False,
                   collective=False, automate=False, anonyme=False, ca=False,
                   weak=False):
        func_vars = vars().copy()
        mask = 0
        for key, value in Certificate.auth_method.iteritems():
            mask += value if func_vars.get(key) else 0
        return mask



import os
from setuptools import setup, find_packages


def README():
    try:
        import pypandoc
        return pypandoc.convert('README.md', 'rst')
    except (IOError, ImportError):
        return open('README.md').read()

setup(
    name='django-idshost',
    version='0.1.0',
    packages=find_packages(),
    include_package_data=True,
    license='Apache License 2.0',
    description='Django app to interface with the ids infrastructure.',
    long_description=README(),
    url='https://github.com/optiflows/django-idshost',
    author='Valentin Monte',
    author_email='valentin.monte@optiflows.com',
    install_requires=[
        'django>=1.5',
        'spyne>=2.11,<3.0',
        'suds>=0.4, <0.5',
        'lxml>=3',
    ],
    setup_requires=[
        'setuptools_git>=1.0',
        'pypandoc',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
